; ---------------------------------------------------------------------------------------------------------------
; EDT3259 - 2019-11-18 - Pere Serrat 	-	Allow modifying colours in CoCo-VGA module
; ---------------------------------------------------------------------------------------------------------------
; MEMORY MAP for this app:
; $1000-$11ff 	- page to be sent to CoCo-VGA
; $1300-$1339	- palette structure
; $1400-$1c00	- screen for mode SG64
; $2001-$27ff	- Basic ... we must issue a PCLEAR3 to move Basic to $1e00 ($2000 for CoCo)
; $2800-$3fff	- program to run in 16k machines --> $1800 = 6144d
; ---------------------------------------------------------------------------------------------------------------
			ORG	$2800
			PUT	$2800
; ---------------------------------------------------------------------------------------------------------------
; structure for VGA config area and structure for AMC2 boards. MUST begin in a 512 bytes boundary address
; ---------------------------------------------------------------------------------------------------------------
vgaRegs	equ	$1000							; $88 bytes - area to be used to configure VGA. Mandatory for D32
SLOTRST	equ	$00							; offset for reset
SLOTEDT	equ	$01							; offset for edit mask
SLOTFNT	equ	$03							; offset for font
SLOTART	equ	$04							; offset for artifact
SLOTEXT	equ	$05							; offset for extras
SLOTENH	equ	$08							; offset for enhanced modes
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SLOTBLK	equ	$20							; offset substitute for color BLACK
SLOTGRN	equ	$22							; offset substitute for color GREEN
SLOTYLW	equ	$24							; offset substitute for color YELLOW
SLOTBLU	equ	$26							; offset substitute for color BLUE
SLOTRED	equ	$28							; offset substitute for color RED
SLOTBUF	equ	$2a							; offset substitute for color BUFF
SLOTCYA	equ	$2c							; offset substitute for color CYAN
SLOTMAG	equ	$2e							; offset substitute for color MAGENTA
SLOTORG	equ	$30							; offset substitute for color ORANGE
SLOTDGR	equ	$32							; offset for color dark Green
SLOTDOG	equ	$34							; offset for color dark Orange
SLOTFGT	equ	$80							; offset for text Foreground 
SLOTBGT	equ	$82							; offset for text Background 
SLOTBOR	equ	$84							; offset for Border 
SLOTSCL	equ	$86							; offset for Scanlines
VGALEN	equ	$88							; area length
; ---------------------------------------------------------------------------------------------------------------
; structure for palete data. The palete file *MUST* be loaded here from disk in case we chose that way!
; ---------------------------------------------------------------------------------------------------------------
wrkPal	equ	$1300							; 30 bytes reserved to read-write palete file
wrkNam	equ	$131E							; game name (8 chars)
wrkDesc	equ	$1326							; game description (20 bytes)
palEnd	equ	$1339							; end of area
palLen	equ	palEnd-wrkPal				; value for clearing area (58 bytes)
; - - - - - - - -	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PALBLK	equ	00								; 2 bytes for color BLACK
PALGRN	equ	02								; 2 bytes for color GREEN
PALYLW	equ	04								; 2 bytes for color YELLOW
PALBLU	equ	06								; 2 bytes for color BLUE
PALRED	equ	08								; 2 bytes for color RED
PALBUF	equ	10								; 2 bytes for color BUFF
PALCYA	equ	12								; 2 bytes for color CYAN
PALMAG	equ	14								; 2 bytes for color MAGENTA
PALORG	equ	16								; 2 bytes for color ORANGE
PALBOR	equ	18								; 2 bytes for color BORDER
PALDGR	equ	20								; 2 bytes for color dark Green
PALDOG	equ	22								; 2 bytes for color dark Orange
PALFGT	equ	24								; 2 bytes for text Foreground 
PALBGT	equ	26								; 2 bytes for text Background 
PALSCL	equ	28								; 2 bytes for Scanline
PALDUM	equ	30								; 2 unused bytes to round up count
													; for a total length of 32 bytes
; ---------------------------------------------------------------------------------------------------------------
INTDIS	equ	$50							; value to disable interrupts
INTENA	equ	$af							; value to enable interrupts
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BASBEG	equ	$19							; basic beginning area
CHRGET	equ	$9f							; system char get routine
CHRGPTR	equ	$a6							; char get pointer
SCRBEG	equ	$0400							; default text screen beginning
RCOMPBEG	equ	SCRBEG+32					; trick to point to R component value
LASTLINE	equ	SCRBEG+$1e1					; point to 2nd char of last screen line
SCREND	equ	$0600							; default text screen end
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	IF COCO
TOKENCMD	equ	$b821							; tokenize command
EXECCMD	equ	$adc6							; execute requested command
	ELSE
TOKENCMD	equ	$8f67							; tokenize command
EXECCMD	equ	$84dc							; execute requested command
	ENDIF
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
READKEYB	equ	$A000							; read keyboard
PIA0BCTL	equ	$ff03							; PIA0 side B control register
PIA1BDAT	equ	$ff22							; PIA1 sideB data register
SAMADDRS	equ	$ffc6							; 1st address for SAM to define working page in RAM
; ===============================================================================================================
PgmBeg	lbsr 	CopyScr						; show main menu on screen
			clr	curCcs						; set CCS0
			clr	datChg						; no changes done
			lbsr	CleanVga						; clean VGA area
			lbsr	RstModul						; ask CoCo-VGA to reset to default values
ReIntro	lbsr	UpdRGBs						; update RGB values on screen to synchronize data <-> screen
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			ldx	#elemOff						; point to offset table
			ldb	#1								; point to first colour element (after black)
			stb	elemen						; save into variable
			lbsr	DoChng1						; invert first element
			ldx	#menuMain					; point to main menu
			lbsr	PrtMenu						; show it
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
LKey1		ldb	elemen						; get element number
			lda	PIA1BDAT						; get current palette configuration
			cmpb	#11							; was item CSS1?
			beq	SetCss1						; yes, set it
			cmpb	#10							; was item CSS0?
			bne	LKey11						; no, go to keypress control
SetCss0	tst	curCcs						; already in Css0 mode?
			beq	LKey11						; yes, go read keyboard
			anda	#%11110111					; clear bit3 to force colour 0
			clr	curCcs						; reset variable to CCS0
			bra	SetCss						; skip next
SetCss1	tst	curCcs						; already in Css1 mode?
			bne	LKey11						; yes, go read keyboard
			ora	#%00001000					; set bit3 to force colour 1
			ldb	#1								; put this vaue
			stb	curCcs						; to set CCS1
SetCss	sta	PIA1BDAT						; update palette configuration
LKey11	jsr	[READKEYB]					; wait for a keypress
			beq	LKey11						; none? loop
			ldb	elemen						; get current element number
			cmpa	#9								; pressed right?
			lbeq	NRight						; yes, process it
			cmpa	#8								; pressed left?
			lbeq	NLeft							; yes, process it
			cmpa	#94							; pressed up?
			lbeq	NUp							; yes, process it
			cmpa	#10							; pressed down?
			lbeq	NDown							; yes, process it
			cmpa	#'Q'							; was command Exit?
			beq	OpQuit						; yes, process it
			cmpa	#'E'							; was command Edit?
			lbeq	OpEdt							; yes, process it
			cmpa	#'L'							; was command Load?
			beq	LdFile						; yes, exit
			cmpa	#'S'							; was command Save?
			beq	SvFile						; yes, process it
			cmpa	#'D'							; was command Dir?
			beq	DskDir						; yes, process it
			cmpa	#'R'							; was command Reset?
			beq	RstVga						; yes, process it
			bra	LKey11						; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
DskDir	ldx	#dirTxt						; point to DIR command text
			lbsr	CpyCmd						; copy to command string
			lbsr	SendCmd						; send command to system
			ldx	#doneW						; point to message for the user
			lbsr	ShowTxt						; show it
			lbsr	WaitKey						; wait for a keypress
			lbsr 	CopyScr						; show main menu on screen
			lbra	ReIntro						; redraw main screen
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
OpQuit	bsr	UsrShure						; tell user changes not saved
			cmpa	#'B'							; was answer command back? 
			beq	GoBack						; yes, process it
			bra	PgmEx01						; else exit pgm
GoBack	ldx	#menuMain					; point to main menu
			lbsr	PrtMenu						; show it
			bra	LKey11						; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
UsrShure	lda	#'C'							; continue by default
			tst	datChg						; are there pending changes (not yet saved)?
			beq	UsShuEx						; no, exit immediately
			ldx	#menuQuit					; point to menu quit
			lbsr	PrtMenu						; show it
LKeyQ		jsr	[READKEYB]					; wait for a keypress
			beq	LKeyQ							; none? loop
			cmpa	#'B'							; is command back?
			beq	UsShuEx						; yes,
			cmpa	#'C'							; is command continue?
			bne	LKeyQ							; no, reject command and loop
UsShuEx	rts									; return			
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PgmEx01	ldx	<CHRGPTR						; get interpreter pointer
			ldd	,x								; get address here
			cmpd	<BASBEG						; is it lower than Basic start?
			blo	NoChange						; yes, skip adjustement
			clra									; null byte
			sta	,x								; clean 2 bytes to avoid
			sta	1,x							; SN error returning to Basic
NoChange	rts									; return to caller (basic program)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
LdFile	bsr	UsrShure						; tell user changes not saved
			cmpa	#'B'							; was answer command back? 
			beq	GoBack						; yes, go back for next keypress
			lbsr	LoadFile						; load palete from disk
			clr	datChg						; no changes made
			lbra	LKey11						; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SvFile	lbsr	SaveFile						; save palete to disk
			clr	datChg						; no changes made
			lbra	LKey11						; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
RstVga	bsr	UsrShure						; tell user changes not saved
			cmpa	#'B'							; was answer command back? 
			beq	GoBack						; yes, go back for next keypress
			lbsr	CleanVga						; clean VGA area
			lbsr	RstModul						; ask CoCo-VGA to reset to default values
			lbsr	UpdRGBs						; update RGB values on screen
			lda	#1								; set to true
			sta	datChg						; changes made variable
			bra	GoBack						; to show previous menu
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
OpEdt		lbsr	EditCol						; call menu Edit
			lbra	LKey1							; loop to read keyborad again verifying CSS
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
NDown		cmpb	#9								; is current element already in the last row?
			lbhi	LKey11						; yes, reject command
			addb	#5								; point to element down on next row
			bra	DoChange						; update titles
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
NUp		cmpb	#5								; is current element already in the first row?
			lblo	LKey11						; yes, reject command
			subb	#5								; point to element up on previous row
			bra	DoChange						; update titles
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
NLeft		tstb									; is current element already the first one?
			lbeq	LKey11						; yes, reject command
			decb									; point to previous element
			bra	DoChange						; update titles
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
NRight	cmpb	#14							; is current element already the last one?
			lbhs	LKey11						; yes, reject command
			incb 									; point to next element
; ---------------------------------------------------------------------------------------------------------------
; first restore old element title. Use regB as new element and invert title
; ---------------------------------------------------------------------------------------------------------------
DoChange	bsr	Change						; to allow for multi-use
			lbra	LKey1							; loop to read keyboard again verifying CSSn
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Change	pshs	b								; save requested item number
			ldx	#elemOff						; point to offset table
			ldb	elemen						; get current element number
			bsr	InvElem						; invert title
			puls	b								; get requested number
DoChng1	stb	elemen						; update variable
			bsr	InvElem						; invert title
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; invert title on current colour element. regB = element number
; ---------------------------------------------------------------------------------------------------------------
InvElem	aslb									; calculate table offset
			ldd	b,x							; get offset for requested item
			ldu	#SCRBEG						; point to screen beginning
			leau	d,u							; point to item tittle
			stu	titlePtr						; save address
			ldb	#5								; number of chars to invert
DoCh01	lda	,u								; get a char
			eora	#64							; invert upper <-> lower
			sta	,u+							; update screen
			decb									; decrement counter
			bne	DoCh01						; not done? loopback
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; subroutine to print a whole message pointed by regX
; ---------------------------------------------------------------------------------------------------------------
PrtMenu	ldu	#LASTLINE					; point to menu screen line
PrntStr	lda	,x+							; get a char from string
			beq	PrntEx01						; if got to the end, exit
			cmpa	#124							; is it normal <?
			beq	Prnt1							; yes, print it
			cmpa	#126							; is it normal >?
			beq	Prnt1							; yes, print it
			cmpa	#96							; is it lowercase?
			ble	Prnt1							; no, skip next
			anda	#%10011111					; convert ASCII to POKE value
Prnt1		sta	,u+							; put on screen
			bra	PrntStr						; go for next char
PrntEx01	rts									; return			
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
; load file from disk to palette and VGA areas
; ---------------------------------------------------------------------------------------------------------------
LoadFile	lbsr	CleanVga						; clean VGA area
			lbsr	LoadPal						; read file to palette area
			lbsr	Pal2Vga						; copy data to VGA area
			ldx	#vgaRegs						; point to VGA area
			lda	#$2c							; set bits 5,3,2 
			sta	SLOTEDT,x					; in param
			lda	SLOTEXT,x					; get 'extras' param
			tst	SLOTBOR,x					; is border enabled?
			bpl	NoBorder						; no, skip next one
			ora	#$03							; enable border color			
NoBorder	ora	#$08							; enable quiet border status
			tst	SLOTSCL,x					; are scanlines enabled?
			bpl	NoScanL						; no, skip next two
			ora	#$04							; enable scanlines
NoScanL	tst	SLOTFGT,x					; is foreground text enabled?
			bne	EnaTxtC						; yes, enable it
			tst	SLOTBGT,x					; is background text enabled?
			beq	LfEna							; no, skip next
EnaTxtC	ora	#$80							; enable text colors
LfEna		sta	SLOTEXT,x					; update param
			lbsr	UpdRGBs						; update RGB values on screen to synchronize data <-> screen
			lbsr	ApplyMod						; send page to module
LdFilEx	ldx	#menuMain					; point to main menu
			bsr	PrtMenu						; show it
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; save VGA area to palete and the to disk
; ---------------------------------------------------------------------------------------------------------------
SaveFile	bsr	CleanPal						; clean palette area
			lbsr	Vga2Pal						; copy VGA data to palette
			lbsr	SavePal						; save file from palette area
			bra	LdFilEx						; restore main menu
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; clean palette area
; ---------------------------------------------------------------------------------------------------------------
CleanPal	ldx	#wrkPal						; point to palette beginning
			clra									; zero byte
			ldb	#palLen						; palette length
ClPa01	sta	,x+							; clean on ebyte
			decb									; decrement counter
			bne	ClPa01						; not done? loopback
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; menu for editing options
; ---------------------------------------------------------------------------------------------------------------
EditCol	ldx	#menuEdt						; point to menu for editting colors
			bsr	PrtMenu						; show it
			lda	#1								; set 1st component
			sta	edtColN						; being edited
MovComp	lbsr	InvEdtN						; invert 1st component
LKey2		jsr	[READKEYB]					; wait for a keypress
			beq	LKey2							; none? loop
			cmpa	#94							; pressed component up?
			beq	OpCpUp						; yes, process it
			cmpa	#10							; pressed component down?
			beq	OpCpDwn						; yes, process it
			cmpa	#'>'							; pressed > (shifted)
			beq	OpRight						; yes, process it
			cmpa	#'.'							; pressed > (NOT shifted)
			beq	OpRight						; yes, process it
			cmpa	#'<'							; pressed < (shifted)
			beq	OpLeft						; yes, process it
			cmpa	#','							; pressed < (NOT shifted)
			beq	OpLeft						; yes, process it
			cmpa	#'O'							; was command Out (go Back)?
			beq	EdColEx						; exit updating menu line
			cmpa	#'D'							; was command disable this colour change?
			beq	OpDis							; yes, process it
			cmpa	#'E'							; was command enable this colour change?
			beq	OpEna							; yes, process it
			bra	LKey2							; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
OpCpUp	lda	edtColN						; get current component
			cmpa	#1								; already at 1st one?
			beq	LKey2							; yes, reject commad
			bsr	InvEdtN						; invert current component value
			dec	edtColN						; go one up
			bra	MovComp						; invert requested one
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
OpCpDwn	lda	edtColN						; get current component
			cmpa	#3								; already at 1st one?
			beq	LKey2							; yes, reject commad
			bsr	InvEdtN						; invert current component value
			inc	edtColN						; go one down
			bra	MovComp						; invert requested one
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
OpRight	bsr	GetCurV						; get current value
			cmpa	#7								; has already max value?
			beq	LKey2							; yes, reject commad
			inca									; increment value
			bsr	UpdCurV						; update value on screen and VGA area
			bra	LKey2							; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
OpLeft	bsr	GetCurV						; get current value
			tsta									; has already min value?
			beq	LKey2							; yes, reject commad
			deca									; decrement value
			bsr	UpdCurV						; update value on screen and VGA area
			bra	LKey2							; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
OpDis		lda	#4								; for 'd'
			bra	PutLetr						; skip next
OpEna		lda	#5								; for 'e'
PutLetr	ldx	titlePtr						; point to item title
			cmpa	100,x							; already same state?
			beq	OpDisEx						; yes, exit
			sta	100,x							; update status on screen
			lbsr	UpdSta						; update 1st byte in VGA area for this item to show its new state (bit 7)
			lbsr	ApplyMod						; send page to module to update color changes
			lda	#1								; set to true
			sta	datChg						; changes done variable
OpDisEx	bra	LKey2							; loop to read keyboard again
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EdColEx	bsr	InvEdtN						; invert current number being edited
			ldx	#menuMain					; point to main menu
			lbsr	PrtMenu						; show it
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; return current char (poke value) in regA and offset to regX in regB. Input Param in edtColN
; ---------------------------------------------------------------------------------------------------------------
GetChrV	clrb									; offset
			lda	edtColN						; get component being edited
GetCh01	addb	#32							; jump to next component
			deca									; decrement counter
			bne	GetCh01						; not done? loop
			ldx	titlePtr						; get pointer to item tittle
			lda	b,x							; get current value
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; return current component value in regA and offset to regX in regB. Input Param in edtColN
; ---------------------------------------------------------------------------------------------------------------
GetCurV	bsr	GetChrV						; get current char
			anda	#7								; use lower three bits
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; update current component value in regA and update VGA area accordingly
; ---------------------------------------------------------------------------------------------------------------
UpdCurV	pshs	a								; save new value
			bsr	GetChrV						; get current offset at regB and address at regX
			puls	a								; restore new value
			ora	#%01110000					; force non inverted number
			sta	b,x							; update screen
			bsr	UpdCmp						; update VGA area accordingly
			lbsr	ApplyMod						; send page to module to update color changes
			lda	#1								; set to true
			sta	datChg						; data changed variable
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; invert item figure on screen
; ---------------------------------------------------------------------------------------------------------------
InvEdtN	bsr	GetChrV						; get current poke value
			eora	#64							; toggle number normal-inverted
			sta	b,x							; update screen
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; update component data in VGA area for an item
; input params: 	regA - value to be saved
; 						regB - offset to current component being edited: 32-64-96 -> R G B
; ---------------------------------------------------------------------------------------------------------------
UpdCmp	anda	#7								; ensure use of 3 lower bits only (values 0-7)
			pshs	a,b							; save new value and current offset to screen value
			ldx	#vgaOff						; point to VGA offsets table
			ldb	elemen						; get item number being processed
			ldb	b,x							; get offset to requested item
			clra									; make 16 bits offset
			ldx	#vgaRegs						; point to VGA area beginning
			leax	d,x							; point to 1st byte for requested colour item
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			ldb	1,s							; get received offset
			cmpb	#32							; was for R?
			beq	UpdR							; yes, process R
			cmpb	#64							; was for G?
			beq	UpdG							; yes, process G
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
UpdB		lda	1,x							; get current 2nd byte value
			anda	#%11100011					; clear B value	
			sta	1,x							; save cleaned value
			lda	,s								; get new value
			asla									; move 3 lower bits
			asla									; to bits 4-3-2
			adda	1,x							; add cleaned value
			sta	1,x							; update 2nd byte
			bra	UpdEx							; exit cleaning stack
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
UpdG		ldd	,x								; get both bytes
			anda	#%11111100					; clear
			andb	#%01111111					; G value
			std	,x								; save cleaned values
			lda	,s								; get new value
			clrb									; clean 2nd byte
			lsra									; move bits 2-1 to bits 1-0, bit 0 goes to carry
			rorb									; receive carry (bit0) into bit7
			addd	,x								; add cleaned values
			std	,x								; update both bytes
			bra	UpdEx							; exit cleaning stack
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
UpdR		lda	,x								; get current 1st byte value
			anda	#%10001111					; clear R value
			sta	,x								; save cleaned value
			lda	,s								; get new value
			asla									; move 3
			asla									; lower bits
			asla									; to bits
			asla									; 6-5-4
			adda	,x								; add cleaned value
			sta	,x								; update 1st byte
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
UpdEx		leas	2,s							; discard pushed values	
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; update state in VGA area for an item
; State is bit7 of first data byte for any colour item
; regA comes with value '4' - '5' designing Disable - Enable
; ---------------------------------------------------------------------------------------------------------------
UpdSta	pshs	a								; save parameter
			ldx	#vgaOff						; point to offsets table
			ldb	elemen						; get item number being processed
			ldb	b,x							; get offset to requested item
			clra									; make 16 bits offset
			ldx	#vgaRegs						; point to VGA area beginning
			leax	d,x							; point to 1st byte
			ldb	,x								; get it
			lda	,s								; get parameter from stack
			cmpa	#4								; is actual status Disable?
			beq	UpdDis						; yes, process it
UpdEna	orb	#%10000000					; set bit7
			bra	UpdUpd						; skip next
UpdDis	andb	#%01111111					; reset bit7			
UpdUpd	stb	,x								; update 1st byte in VGA area
												; params that must be changed upon element being modified
			puls	a								; restore param from stack
			cmpa	#4								; is it disable?
			bne	UpEnaPar						; no, process enable parameters
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
UpDisPar	ldx	#vgaRegs						; point to VGA data area
			ldu	#vgaOff						; point to vga offsets table
			ldb	elemen						; get element being processed
			cmpb	#9								; is it border?
			beq	DisBder						; yes, process it
			cmpb	#12							; is it foreground text?
			beq	DisFGT						; yes, process it
			cmpb	#13							; is it background text?
			beq	DisBGT						; yes, process it
			cmpb	#14							; is it scanlines?
			bne	UpDisEx						; no, exit
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
DisScan	lda	SLOTEXT,x					; get 'extras' param
			anda	#$fb							; reset bit2
			sta	SLOTEXT,x					; update param
			rts									; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
DisBder	lda	SLOTEXT,x					; get 'extras' param
			anda	#$fc							; reset bits1-0
			sta	SLOTEXT,x					; update param
			rts									; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
DisFGT	incb									; index to background text
			fcb	$86							; skips next one converting to: lda #$5a
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
DisBGT	decb									; index to foreground text
			ldb	b,u							; get offset for background text
			clra									; make 16 bits offset
			ldb	d,x							; get 1st byte for that element
			bitb	#$80							; is bit7 set?
			bne	UpDisEx						; yes, exit
			lda	SLOTEXT,x					; get 'extras' param
			anda	#$7f							; reset bit7
			sta	SLOTEXT,x					; update param
UpDisEx	rts
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
UpEnaPar	ldx	#vgaRegs						; point to VGA data area
			ldb	elemen						; get element being processed
			cmpb	#9								; is it border?
			beq	EnaBder						; yes process it
			cmpb	#14 							; is it scanlines?
			beq	EnaScan						; yes process it
			cmpb	#12							; is it lower than foreground?
			blo	EnaCol						; yes process color
			beq	EnaTxt						; yes process foreground
			cmpb	#13							; is it background text?
			bne	UpEnaEx						; no, exit
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EnaTxt	lda	SLOTEDT,x					; get 'edit mask' param
			ora	#$24							; set bits 5 and 2 for extra palette and extras
			sta	SLOTEDT,x					; update value
			lda	SLOTEXT,x					; get 'extras' param
			ora	#$80							; set bit 7 for text palette enable
			sta	SLOTEXT,x					; update value
			rts									; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EnaCol	lda	SLOTEDT,x					; get 'edit mask' param
			ora	#$08							; set bit 3 for semigraphics palette
			sta	SLOTEDT,x					; update value
			rts									; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EnaScan	lda	SLOTEDT,x					; get 'edit mask' param
			ora	#$24							; set bits 5 and 2 for extra palette and extras
			sta	SLOTEDT,x					; update value
			lda	SLOTEXT,x					; get 'extras' param
			ora	#$04							; set bit 2 for scanline enable
			sta	SLOTEXT,x					; update value
			rts									; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
EnaBder	lda	SLOTEDT,x					; get 'edit mask' param
			ora	#$24							; set bits 5 and 2 for extra palette and extras
			sta	SLOTEDT,x					; update value
			lda	SLOTEXT,x					; get 'extras' param
			ora	#$03							; set bits 0-1 for border color mode
			sta	SLOTEXT,x					; update value
UpEnaEx	rts									; return
; ---------------------------------------------------------------------------------------------------------------
; copies the default menu screen to the std text area $400-$5ff
; ---------------------------------------------------------------------------------------------------------------
CopyScr	ldx	#SCRBEG						; point to destination
			ldu	#mainMenu					; point to default screen
SS01		ldd	,u++							; get 2 bytes
			std	,x++							; put at destination
			cmpx	#SCREND						; got to screen end?
			blo	SS01							; no, loopback
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; send to CoCo-VGA a page with just the reset byte to $ff
; ---------------------------------------------------------------------------------------------------------------
RstModul	lda	#$ff							; set all features to be set to default values
			sta	vgaRegs						; put in VGA area
			lbsr	ApplyMod						; send page to module
			bsr	CleanVga						; reset VGA area all bytes
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; clear (reset) all bytes in VGA area
; ---------------------------------------------------------------------------------------------------------------
CleanVga	ldx	#vgaRegs						; point to area beginning
			ldb	#VGALEN						; get area length
			clra									; get a zero
ClnV1		sta	,x+							; clean one byte
			decb									; decrement counter
			bne	ClnV1							; not done? loopback
			ldx	#vgaRegs						; point to VGA area
			clr	,x								; clear reset byte
			clr	SLOTEDT,x					; clear edit mask byte
			ldd	#$0D48						; regA: 00001110 -> artifacts
													; regB: 01001000 -> text palette,quiet border,border color mode
			std	SLOTART,x					; put in VGA area
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; update RGB values for all 15 elemnts in screen
; ---------------------------------------------------------------------------------------------------------------
UpdRGBs	clrb									; get a zero
UpRGB1	pshs	b								; save counter
												; point to 1st data value (for R)
			ldx	#elemOff						; point to offset table
			aslb									; calculate table offset
			ldd	b,x							; get offset (2 bytes) for requested item
			ldu	#RCOMPBEG					; ready to point to R value on screen
			leau	d,u							; point to value for R
			stu	rgbPtr						; save working address
												; point to 2 bytes for these item in VGA area
			ldx	#vgaOff						; point to VGA offsets table
			ldb	,s								; get counter from stack
			ldb	b,x							; get offset (1 byte) for this item
			clra									; make 16 bits offset
			ldu	#vgaRegs						; point to area beginning
			ldd	d,u							; get both bytes
			bsr	Extract						; extract and show R,G,B data from them
			puls	b								; get counter
			incb									; increment it
			cmpb	#15							; processed last one?
			blo	UpRGB1						; no, loopback for next one
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; decode regD to show values RGB for one screen colour item
; ---------------------------------------------------------------------------------------------------------------
Extract	ldu	rgbPtr						; point to R data
			std	datRGB						; store received values for later use
			tsta									; check bit7
			bmi	IsEnabld						; if set skip next two
			ldb	#4								; value for 'd'
			bra	Ext01							; skip next
IsEnabld	ldb	#5								; value for 'e'
Ext01		stb	68,u							; show value enabled/disabled
			lsra									; move R data
			lsra									; to the
			lsra									; lower 
			lsra									; nibble
			anda	#7								; use only 3 bits
			adda	#48							; to convert to inverted number
			sta	,u								; show component R
			ldd	datRGB						; get data again
			rolb									; moves high bit to carry (lower G bit)
			rola									; get carry into bit0
			anda	#7								; use only 3 bits
			adda	#48							; to convert to inverted number
			sta	32,u							; show component G
			lsrb									; move B data
			lsrb									; to the
			lsrb									; lower nibble
			andb	#7								; use only 3 bits
			addb	#48							; to convert to inverted number
			stb	64,u							; show component B
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; copy VGA area to palette structure
; ---------------------------------------------------------------------------------------------------------------
Vga2Pal	ldx	#vgaRegs						; point to VGA source area
			ldu	#wrkPal						; point to destination palette area
			ldy	#vgaOff						; point to VGA offset table
			clr	,-s							; create and reset a counter in stack
Vga2P01	ldb	,s								; get counter from stack
			cmpb	#15							; got to the end?
			bhs	Vga2PEx						; yes, exit
			ldb	b,y							; get VGA offset for this item
			clra									; make 16 bits offset
			ldd	d,x							; get 2 bytes from VGA area (signed addition)
			std	,u++							; copy to palette area and move pointer
			inc	,s								; increment counter
			bra	Vga2P01						; loopback
Vga2PEx	puls	b,pc							; clean stack and return
; ---------------------------------------------------------------------------------------------------------------
; copy palette area to VGA area
; ---------------------------------------------------------------------------------------------------------------
Pal2Vga	ldu	#wrkPal						; point to loaded palette area
			ldy	#vgaOff						; point to VGA offset table
			clr	,-s							; create and reset a counter in stack
Pal2V01	ldb	,s								; get counter from stack
			cmpb	#15							; got to the end?
			bhs	Pal2VEx						; yes, exit
			ldb	b,y							; get VGA offset for this item
			clra									; make 16 bits offset
			ldx	#vgaRegs						; point to VGA beginning area
			leax	d,x							; point to destination item data (unsigned addition)
			ldd	,u++							; get 2 bytes from palette and move pointer 
			std	,x								; copy to VGA area
			inc	,s								; increment counter
			bra	Pal2V01						; loopback
Pal2VEx	puls	b,pc							; clean stack and return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
; subroutines to LOAD and SAVE to/from disk
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
LoadPal	lbsr	CleanCmd						; clean command line
			ldx	#loadFil						; point to LOAD command text
			bsr	Perform						; apply this command
			bsr	LoadAdrs						; add Load address
			bsr	SendCmd						; send command to DOS
			bra	WaitKey						; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SavePal	lbsr	CleanCmd						; clean command line
			ldx	#saveFil						; point to SAVE command text 
			bsr	Perform						; apply this command
			bsr	SaveAdrs						; add Load address
			bsr	SendCmd						; send command to DOS
WaitKey	jsr	[READKEYB]					; wait for keypress
			beq	WaitKey						; none, loop
			ldx	#cleanL						; point to mesage cleaner
			bsr	ShowTxt						; show it
			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
Perform	bsr	CpyCmd						; copy command text to command line
			ldx	#cleanL						; point to message cleaner
			bsr	ShowTxt						; show it
			ldx	#textN						; point to message for user
			bsr	ShowTxt						; show it
			bsr	AskName						; wait for user to enter filename
			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
CpyCmd	ldu	#dosCmd						; point to command line data
	IF COCO
			ldb	#5								; bytes to copy
	ELSE
			ldb	#4								; bytes to copy
	ENDIF
Cpy01		lda	,x+							; get a byte
			sta	,u+							; put in command line
			decb									; decrement counter
			bne	Cpy01							; not done? loop
			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
ShowTxt	ldu	#$05e1						; point to last screen line
ShwT01	lda	,x+							; get a char
			beq	ShwTEx						; if zero, exit
			ora	#$40							; ensure upercase
			sta	,u+							; put on screen
			bra	ShwT01						; go for next char
ShwTEx	rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
AskName	clr	numChar						; counter to zero
			ldx	#dosParms					; point to params area in command line
			bsr	WaitNam						; wait for filename
			ldd	#".P"							; add .PAL to the command
			std	,x++							; advancing
			ldd	#"AL"							; command
			std	,x++							; pointer
			lda	#'"'							; close name 
			sta	,x+							; with quotes
			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
SaveAdrs	ldu	#saveMem						; point to save addresses
			bra	AddMem						; go add it
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
LoadAdrs	bra	Add2CEx						; go add a null
AddMem	bsr	Add2Cmd						; add to command line
			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
Add2Cmd	lda	,u+							; get a char
			beq	Add2CEx						; if zero exit
			sta	,x+							; put in command line
			bra	Add2Cmd						; go for next char
Add2CEx	clr	,x								; add a final zero
			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
SendCmd	orcc	#INTDIS						; disable interrupts
			ldx	<CHRGPTR						; get current system pointer ($A6)
			pshs	x								; save on stack
			leax	dosCmd,PCR					; get command line address
			stx	<CHRGPTR						; pass to system pointer
			andcc	#INTENA						; enable interrupts
			jsr	TOKENCMD						; tokenize command string
			jsr	<CHRGET						; get first token
			jsr	EXECCMD						; dispatch command
			ldx	#doneW						; point to message for the user
			bsr	ShowTxt						; show it
			puls	x								; get system pointer from stack
			stx	<CHRGPTR						; restore it
			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
WaitNam	pshs	x,u							; save pointers (command and screen)
WNam01	jsr	[READKEYB]					; read Keyboard
			beq	WNam01						; no keypress, loopback
			ldx	,s								; get command pointer
			ldu	2,s							; and screen pointer
			cmpa	#13							; enter received?
			beq	WNamEx						; exit
			cmpa	#8								; received backspace?
			bne	WNam02						; no, go to next control
			tst	numChar						; see number of chars received
			beq	WNam01						; if zero no backspace possible, reject keystroke
			dec	numChar						; decrement number of chars
			leax	-1,x							; move command pointer back
			leau	-1,u							; and screen pointer too
			stx	,s								; update pointers
			stu	2,s							; in the stack
			lda	#96							; get a space
			sta	,u								; clean char on screen
			clr	,x								; clean char on command string
			bra	WNam01						; loopback for another keystroke			
WNam02	cmpa	#48							; was a sign lower than '0'?
			blo	WNam01						; yes, reject it
			cmpa	#57							; was a number lower or equal to 9?
			ble	GoodNum						; yes, use it
			cmpa	#65							; was a sign lower than 'A'?
			blo	WNam01						; yes, reject it
			cmpa	#90							; was a sign greater than 'Z'?
			bhi	WNam01						; yes, reject it
GoodNum	sta	,x+							; put char on command line
			ora	#$40							; ensure correct POKE value
			sta	,u+							; put on screen
			stx	,s								; update command pointer
			stu	2,s							; and screen pointer
			inc	numChar						; increment counter
			ldb	numChar						; get char counter
			cmpb	#8								; already 8 chars received?
			blo	WNam01						; no, loop for more
WNamEx	puls	x,u,pc						; restore pointers and return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
CleanCmd	ldx	#dosParms					; point to command line params
			ldb	#parmLen						; number of bytes
			clra									; get a zero
ClCm01	sta	,x+							; clean one byte
			decb									; decrement counter
			bne	ClCm01						; not done? loop
			rts									; return
; ===============================================================================================================
; send page to module CoCo-VGA
; ===============================================================================================================
ApplyMod										; Step 1 - fill VGA area to configure CoCoVGA
													; this step is done by the program before calling this routine
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 2 - Wait for VSYNC, which signals the start of the vertical blanking region.
			pshs	cc     						; save CC
			orcc	#INTDIS  					; mask interrupts
			ldu	#PIA0BCTL					; pointer for PIA0 B-side
			lda	,u	 							; $ff03 - get PIA config
			pshs	a      						; save PIA configuration 
			ora	#$04   						; ensure PIA 0B is NOT setting direction
			sta	,u								; $ff03 - update value
			anda	#$fd   						; vsync flag - trigger on falling edge
			sta	,u								; $ff03 - update value
			ora	#$01   						; enable vsync IRQ (although interrupt itself will be ignored via mask)
			sta	,u								; $ff03 - update value
			lda	-1,u  						; $ff02 - clear any pending VSYNC
L01		lda	,u		  						; $ff03 - wait for flag to indicate...
			bpl	L01     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 3 - During VSYNC, point SAM to 512 byte page set up in step 1 (via SAM page select registers $FFC6-$FFD3)
 			ldd	#vgaRegs						; point to VGA data area beginning
 			bsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 4 - Also during VSYNC, write 6847 VDG combination lock via PIA1B ($FF22) bits 7:3.
Step4		ldy	#PIA1BDAT					; pointer for PIA1 B-side
			lda	,y  							; $ff22 - get current PIA value
			anda	#%00000111					; mask off bits to change
			ora	#%10010000					; $90 - set combo lock 1
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%01001000					; $48 - set combo lock 2
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%10100000					; $a0 - set combo lock 3
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%11111000 					; $f8 - set combo lock 4
			sta	,y	  							; $ff22 - write to PIA
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 5 - Still during VSYNC, configure VDG and SAM back to mode 0 
												; In this case, the desired CoCoVGA register page to program is 0
			lda	,y	  							; $ff22 - get current PIA value
			anda	#%00000111					; mask off bits to change
			ora	#%00000000					; select CoCoVGA register page 0
			sta	,y	  							; $ff22 - write to PIA
			sta	-6,x  						; $ffc0 - clear SAM_V0	0
			sta	-4,x  						; $ffc2 - clear SAM_V1	0
			sta	-2,x  						; $ffc4 - clear SAM_V2	0	-> 000
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 6 -  Wait for next VSYNC while SAM and VDG stream in the entire page of register values 
 												; to CoCoVGA and CoCoVGA displays the previous frame of video
L02		lda	,u								; $ff03 - wait for flag to indicate...
			bpl	L02     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
			puls	a      						; from stack... 
;			sta	-1,u	 						; $ff02 - ...restore original PIA configuration
			sta	,u	 							; $ff03 - ...restore original PIA configuration
			puls	cc     						; restore ability to see interrupts
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 7 - Point SAM page select to video page you want to display
 			ldd	#SCRBEG						; point to text screen beginning
 			bsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 8 - Program SAM and VDG to the appropriate video mode. 
 												; For 32x16 text and graphics, GM is 0 and A/G is 0.
			lda	,y		  						; $ff22 - get current PIA value
			anda	#$0f  						; mask off bits to change
			tst	curCcs						; is current palette 0?
			beq	UseCss0						; yes, skip next two
UseCss1	ora	#%00001000					; set bit3 to ensure color palete 1
			bra	PutCss						; skip next
UseCss0	anda	#%11110111					; reset bit3 to ensure color palete 0
PutCss	sta	,y		  						; $ff22 - write to PIA
			sta	-6,x  						; $ffc0 - clear SAM_V0
			sta	-4,x  						; $ffc2 - clear SAM_V1
			sta	-2,x  						; $ffc4 - clear SAM_V2	-> 000
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; set SAM memory address properly. regD must contain ADDRESS
; ---------------------------------------------------------------------------------------------------------------
SetSAM	ldx	#SAMADDRS					; $ffc6 - pointer for SAM Display Offset
			asra									; use high byte divided by two = regD/512
SamF0		bita	#%00000001					; is bit 0 set?
			bne	SetF0							; yes, setF0
			stb	,x		  						; $c6 - clear 	SAM_F0
			bra	SamF1							; skip next
SetF0		stb	1,x	  						; $c7 - set 	SAM_F0
SamF1		bita	#%00000010					; is bit 1 set?
			bne	SetF1							; yes, setF1
			stb	2,x	  						; $c8 - clear	SAM_F1
			bra	SamF2							; skip next
SetF1		stb	3,x	  						; $c9 - set 	SAM_F1
SamF2		bita	#%00000100					; is bit 2 set?
			bne	SetF2							; yes, setF2
			stb	4,x	  						; $ca - clear 	SAM_F2
			bra	SamF3							; skip next
SetF2		stb	5,x	  						; $cb - set 	SAM_F2
SamF3		bita	#%00001000					; is bit 3 set?
			bne	SetF3							; yes, setF3
			stb	6,x	  						; $cc - clear 	SAM_F3
			bra	SamF4							; skip next
SetF3		stb	7,x	  						; $cd - set 	SAM_F3
SamF4		bita	#%00010000					; is bit 4 set?
			bne	SetF4							; yes, setF4
			stb	8,x	  						; $ce - clear 	SAM_F4
			bra	SamF5							; skip next
SetF4		stb	9,x	  						; $cf - set 	SAM_F4
SamF5		bita	#%00100000					; is bit 5 set?
			bne	SetF5							; yes, setF5
			stb	10,x  						; $d0 - clear 	SAM_F5
			bra	SamF6							; skip next
SetF5		stb	11,x  						; $d1 - set 	SAM_F5
SamF6		bita	#%01000000					; is bit 6 set?
			bne	SetF6							; yes, setF6
			stb	12,x  						; $d2 - clear 	SAM_F6
			rts									; return
SetF6		stb	13,x  						; $d3 - set 	SAM_F6
			rts									; return
; ===============================================================================================================
; offsets for individual slots in VGA area - indexed by item on main menu screen
; ---------------------------------------------------------------------------------------------------------------
vgaOff	fcb	32								; VGA area offset for Black
			fcb	34,36,38,40					; VGA area offset for Green,Yellow,Blue,Red
			fcb	42,44,46,48					; VGA area offset for Buff,Cyan,Magenta,Orange
			fcb	132							; VGA area offset for Border
			fcb	50,52							; VGA area offset for darkGreen,darkOrange
			fcb	128,130,134					; VGA area offset for textForeground,background,scanline
; ---------------------------------------------------------------------------------------------------------------
; offsets for individual slots to their title on menu screen
; ---------------------------------------------------------------------------------------------------------------
elemOff	fdb	$0021,$0027,$002d,$0033,$0039	; first row
			fdb	$00c1,$00c7,$00cd,$00d3,$00d9	; second row
			fdb	$0161,$0167,$016d,$0173,$0179	; 3rd row
; ---------------------------------------------------------------------------------------------------------------
menuMain	fcb	/Quit/,128,/Edit/,128,/Dir/,128,/Load/,128,/Save/,128,/Reset/,128,0
menuEdt	fcb	/Out/,128,/Ena/,128,/Disa/,128,128,/Up Dw col/,128,124,126,/ lv/,128,0
menuQuit fcb	/Back/,128,128,/Cont/,128,128,/CHANGES/,96,/NOT/,96,/SAVED/,0
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
curCcs	fcb	$00							; CCS currently being used
datChg	fcb	$00							; flag for data has changed
datRGB	fdb	$0000							; 2 bytes read from VGA area
edtColN	fcb	$00							; component colour being edited (1-2-3 = RGB)
elemen	fcb	$00							; colour element 
numChar	fcb	$00							; filename char counter
rgbPtr	fdb	$0000							; pointer to RGB data fields
titlePtr	fdb	$0000							; pointer to title 1st char
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; longest command will be: SAVE"FILENAME.PAL",4864,4896,0
; chars needed					     ....+....1....+....2....+ -> 25+zero ending = 26
	IF COCO
dosCmd	fcb	0,0,0,0,0,/"/									; command and leading quotes
dosParms	rzb	26													; area for command parameters
parmLen	equ	loadFil-dosParms
loadFil	fcc	/LOADM/											; load text string
saveFil	fcc	/SAVEM/											; save text string
dirTxt	fcn	/DIR /											; command DIR
	ELSE
dosCmd	fcb	0,0,0,0,/"/										; command and leading quotes
dosParms	rzb	26													; area for command parameters
parmLen	equ	loadFil-dosParms
loadFil	fcc	/LOAD/											; load text string
saveFil	fcc	/SAVE/											; save text string
dirTxt	fcn	/DIR/												; command DIR
	ENDIF
saveMem	fcn	/,4864,4922,0/									; save addresses: $1300, $133A
textN		fcn	/ENTER FILENAME: "/							; message to ask for filename
doneW		fcn	/OPERATION DONE. PRESS KEY/				; ok message
cleanL	fcn	/                             /			; string to clear message line
; ---------------------------------------------------------------------------------------------------------------
	IF COCO
mainMenu	includebin	"VGA3255C.ROM"		; default main menu
	ELSE
mainMenu	includebin	"VGA3255D.ROM"		; default main menu
	ENDIF
; ---------------------------------------------------------------------------------------------------------------
												; this part ensures Drivewire compatibility
	IF COCO
		IF ((*+10)%256 > 0)					; if not multiple of 256
				rzb	 256-((*+10)%256)		; fill to make it DW4 compatible
		ENDIF										; end
	ELSE
		IF ((*+9)%256 > 0)					; if not multiple of 256
				rzb	 256-((*+9)%256)		; fill to make it DW4 compatible
		ENDIF										; end
	ENDIF
; ------------------------------------------------------------------------------------------
PgmEnd	equ	*
