; ---------------------------------------------------------------------------------------------------------------
; RSPAL59 - 2019-11-18 - Pere Serrat 	-	Reset semigraphics and CG palette colors to std values
; ---------------------------------------------------------------------------------------------------------------
			ORG	$4400
			PUT	$4400
; ---------------------------------------------------------------------------------------------------------------
; structure for VGA config area and structure for AMC2 boards. MUST begin in a 512 bytes boundary address
; ---------------------------------------------------------------------------------------------------------------
vgaRegs	equ	$1000							; $88 bytes - area to be used to configure VGA. Mandatory for D32
vgaEnd	equ	$1200							; 1st byte out of that area
SLOTRST	equ	$00							; offset for reset
SLOTEDT	equ	$01							; offset for edit mask
SLOTFNT	equ	$03							; offset for font
SLOTART	equ	$04							; offset for artifact
SLOTEXT	equ	$05							; offset for extras
SLOTENH	equ	$08							; offset for enhanced modes
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SLOTBLK	equ	$20							; offset substitute for color BLACK
SLOTGRN	equ	$22							; offset substitute for color GREEN
SLOTYLW	equ	$24							; offset substitute for color YELLOW
SLOTBLU	equ	$26							; offset substitute for color BLUE
SLOTRED	equ	$28							; offset substitute for color RED
SLOTBUF	equ	$2a							; offset substitute for color BUFF
SLOTCYA	equ	$2c							; offset substitute for color CYAN
SLOTMAG	equ	$2e							; offset substitute for color MAGENTA
SLOTORG	equ	$30							; offset substitute for color ORANGE
SLOTDGR	equ	$32							; offset for color dark Green
SLOTDOG	equ	$34							; offset for color dark Orange
SLOTFGT	equ	$80							; offset for text Foreground 
SLOTBGT	equ	$82							; offset for text Background 
SLOTBOR	equ	$84							; offset for Border 
SLOTSCL	equ	$86							; offset for Scanlines
; ---------------------------------------------------------------------------------------------------------------
INTDIS	equ	$50							; value to disable interrupts
SCRBEG	equ	$0400							; default text screen beginning
PIA0BCTL	equ	$ff03							; PIA0 side B control register
PIA1BDAT	equ	$ff22							; PIA1 sideB data register
SAMADDRS	equ	$ffc6							; 1st address for SAM to define working page in RAM
; ---------------------------------------------------------------------------------------------------------------
												; Step 1 - fill area to configure CoCoVGA
PgmBeg	lbsr	ClearVga						; copy palette data to VGA area
			lda	#$ff							; set all features to be set to default values
			sta	vgaRegs						; put in VGA area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 2 - Wait for VSYNC, which signals the start of the vertical blanking region.
ApplyMod	pshs	cc     						; save CC
			orcc	#INTDIS  					; mask interrupts
			ldu	#PIA0BCTL					; pointer for PIA0 B-side
			lda	,u	 							; $ff03 - get PIA config
			pshs	a      						; save PIA configuration 
			ora	#$04   						; ensure PIA 0B is NOT setting direction
			sta	,u								; $ff03 - update value
			anda	#$fd   						; vsync flag - trigger on falling edge
			sta	,u								; $ff03 - update value
			ora	#$01   						; enable vsync IRQ (although interrupt itself will be ignored via mask)
			sta	,u								; $ff03 - update value
			lda	-1,u  						; $ff02 - clear any pending VSYNC
L01		lda	,u		  						; $ff03 - wait for flag to indicate...
			bpl	L01     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 3 - During VSYNC, point SAM to 512 byte page set up in step 1 (via SAM page select registers $FFC6-$FFD3)
 			ldd	#vgaRegs						; point to VGA data area beginning
 			bsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 4 - Also during VSYNC, write 6847 VDG combination lock via PIA1B ($FF22) bits 7:3.
Step4		ldy	#PIA1BDAT					; pointer for PIA1 B-side
			lda	,y  							; $ff22 - get current PIA value
			anda	#%00000111					; mask off bits to change
			ora	#%10010000					; $90 - set combo lock 1
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%01001000					; $48 - set combo lock 2
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%10100000					; $a0 - set combo lock 3
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%11111000 					; $f8 - set combo lock 4
			sta	,y	  							; $ff22 - write to PIA
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 5 - Still during VSYNC, configure VDG and SAM back to mode 0 
												; In this case, the desired CoCoVGA register page to program is 0
			lda	,y	  							; $ff22 - get current PIA value
			anda	#%00000111					; mask off bits to change
			ora	#%00000000					; select CoCoVGA register page 0
			sta	,y	  							; $ff22 - write to PIA
			sta	-6,x  						; $ffc0 - clear SAM_V0	0
			sta	-4,x  						; $ffc2 - clear SAM_V1	0
			sta	-2,x  						; $ffc4 - clear SAM_V2	0	-> 000
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 6 -  Wait for next VSYNC while SAM and VDG stream in the entire page of register values 
 												; to CoCoVGA and CoCoVGA displays the previous frame of video
L02		lda	,u								; $ff03 - wait for flag to indicate...
			bpl	L02     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
			puls	a      						; from stack... 
			sta	-1,u	 						; $ff02 - ...restore original PIA configuration
			puls	cc     						; restore ability to see interrupts
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 7 - Point SAM page select to video page you want to display
 			ldd	#SCRBEG						; point to text screen beginning
 			bsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 8 - Program SAM and VDG to the appropriate video mode. 
 												; For 32x16 text and graphics, GM is 0 and A/G is 0.
			lda	,y		  						; $ff22 - get current PIA value
			anda	#$0f  						; mask off bits to change
UseCss0	anda	#%11110111					; reset bit3 to ensure color palete 0
PutCss	sta	,y		  						; $ff22 - write to PIA
			sta	-6,x  						; $ffc0 - clear SAM_V0
			sta	-4,x  						; $ffc2 - clear SAM_V1
			sta	-2,x  						; $ffc4 - clear SAM_V2	-> 000
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; set SAM memory address properly. regD must contain ADDRESS
; ---------------------------------------------------------------------------------------------------------------
SetSAM	ldx	#SAMADDRS					; $ffc6 - pointer for SAM Display Offset
			asra									; use high byte divided by two = regD/512
SamF0		bita	#%00000001					; is bit 0 set?
			bne	SetF0							; yes, setF0
			stb	,x		  						; $c6 - clear 	SAM_F0
			bra	SamF1							; skip next
SetF0		stb	1,x	  						; $c7 - set 	SAM_F0
SamF1		bita	#%00000010					; is bit 1 set?
			bne	SetF1							; yes, setF1
			stb	2,x	  						; $c8 - clear	SAM_F1
			bra	SamF2							; skip next
SetF1		stb	3,x	  						; $c9 - set 	SAM_F1
SamF2		bita	#%00000100					; is bit 2 set?
			bne	SetF2							; yes, setF2
			stb	4,x	  						; $ca - clear 	SAM_F2
			bra	SamF3							; skip next
SetF2		stb	5,x	  						; $cb - set 	SAM_F2
SamF3		bita	#%00001000					; is bit 3 set?
			bne	SetF3							; yes, setF3
			stb	6,x	  						; $cc - clear 	SAM_F3
			bra	SamF4							; skip next
SetF3		stb	7,x	  						; $cd - set 	SAM_F3
SamF4		bita	#%00010000					; is bit 4 set?
			bne	SetF4							; yes, setF4
			stb	8,x	  						; $ce - clear 	SAM_F4
			bra	SamF5							; skip next
SetF4		stb	9,x	  						; $cf - set 	SAM_F4
SamF5		bita	#%00100000					; is bit 5 set?
			bne	SetF5							; yes, setF5
			stb	10,x  						; $d0 - clear 	SAM_F5
			bra	SamF6							; skip next
SetF5		stb	11,x  						; $d1 - set 	SAM_F5
SamF6		bita	#%01000000					; is bit 6 set?
			bne	SetF6							; yes, setF6
			stb	12,x  						; $d2 - clear 	SAM_F6
			rts									; return
SetF6		stb	13,x  						; $d3 - set 	SAM_F6
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; routine to clean VGA data area
; ---------------------------------------------------------------------------------------------------------------
ClearVga	ldx	#vgaRegs						; point to VGA data area
			clra									; set a zero
			clrb									; 16 bits long
L03		std	,x++							; reset a word
			cmpx	#vgaEnd						; got to the end?
			blo	L03							; no, loopback
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
												; this part ensures Drivewire compatibility
	IF COCO
		IF ((*+10)%256 > 0)					; if not multiple of 256
				rzb	 256-((*+10)%256)		; fill to make it DW4 compatible
		ENDIF										; end
	ELSE
		IF ((*+9)%256 > 0)					; if not multiple of 256
				rzb	 256-((*+9)%256)		; fill to make it DW4 compatible
		ENDIF										; end
	ENDIF
; ------------------------------------------------------------------------------------------
PgmEnd	equ	*
