; ----------------------------------------------------------------------------------------
; PROGRAM TO EDIT THE FONTS OF AN VGA CHARSET
; FONTEDT v0.1  - PERE SERRAT 2019-11-12
; ----------------------------------------------------------------------------------------
; PRE-REQUISITES:
; the character set (xxxx.CHR) has to be loaded at $0e00-$1a00
; ----------------------------------------------------------------------------------------
; Memory MAP
; 0000-0bff - used by system and DOS
; 0c00-0dff - used by CoCo DOS
; 0e00-19ff - reserved place for the character set
; 1a00-1dff - free
; 1e00-35ff - screen PMODE4, implies PCLEAR8 for Basic use
; 3600-3fff - for Basic: charset loader, ml starter and charset saver
; 4000 		- machine code program
; ----------------------------------------------------------------------------------------
			org	$4000
			put	$4000
; ----------------------------------------------------------------------------------------
SCRNBEG	equ	$1e00							; Screen Beginning
SCREND	equ	$3600							; 1st byte after graphic PMODE4 screen
SCR23L	equ	$3480							; Upper left byte for line 23
PRINTCH	equ	$800f							; std send char to printer
STRING	equ	$90e5							; Basic Print String
SETPIA	equ	$ff22							; PIA
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
FONTAB	equ	$0e00							; Charset table
; ----------------------------------------------------------------------------------------
PgmBeg	lbsr	SetGMode						; set PMODE4,1 - SCREEN 1,1
			clr	edCurY						; reset Y for cursor
 			clr	edCurX						; reset X for cursor
Repaint	bsr	ShowFnts						; paint the 256 chars on screen (16 rows of 16 chars)
			clr	invFlag						; set not inverted
WaitKey	lbsr	ShwPFnt						; show Position and Block number
WforK1	lbsr	Invert						; invert char at cursor
			jsr	[$A000]						; read keyboard
			beq	WforK1						; wait for Key
			cmpa	#3								; Break pressed?
			beq	ExitPgm						; yes, exit
 			cmpa	#9								; requested RIGHT?
 			beq	MovRght						; process right
 			cmpa	#8								; requested LEFT?
 			beq	MovLeft						; process left
 			cmpa	#94							; requested UP?	
 			lbeq	MovUp							; process up
 			cmpa	#10							; requested DOWN?
 			lbeq	MovDown						; process down
			cmpa	#'E'							; requested EDIT mode?
			bne	WforK1						; no, wait for another key
EMode		lbsr	CondInv						; invert font if already inverted
			lbsr	FntEdi4						; editPM4
			bra	Repaint						; loopback to repaint fonts
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
ExitPgm	andcc	#$af							; enable interrupts
			rts									; return
; ----------------------------------------------------------------------------------------
; OK - Draws the whole charset on screen: 8 rows of 32 chars
; ----------------------------------------------------------------------------------------
ShowFnts	ldu	#SCRNBEG						; point to 1st screen byte
			ldx	#FONTAB						; point to 1st byte of 1st font		
NxtRow	ldb	#32							; number of fonts per row
			pshs	b								; onto stack
NxtFnt	pshs	u								; save screen pointer
			ldb	#12							; bytes per character
NxtByte	lda	,x+							; get a byte from charset
			sta	,u								; put on screen
			leau	32,u							; point to row below
			decb									; decrement counter
			bne	NxtByte						; not yet done? loopback
			puls	u								; get screen pointer from stack
			leau	1,u							; point to next byte (next char)
			dec 	,s								; decrement chars per row counter
			bne	NxtFnt						; not all done? loppback
			leas	1,s							; discard counter from stack
			cmpu	#SCRNBEG+2688+32			; have we already worked the eighth row? (32*12x7 + one row
			bhs	ShowFEx						; yes, exit
			leau	384-32,u						; point to next characters row (32*12 - one row)
			bra	NxtRow						; do next chars row
ShowFEx	rts									; return 
; ----------------------------------------------------------------------------------------
; OK - Moves to next char on the right
; ----------------------------------------------------------------------------------------
MovRght	bsr	CondInv						; invert if still inverted
 			lda	edCurX						; get current posX
 			cmpa	#31							; already at the right end?
 			blo	DoRght						; no, do move right
 			clra									; get 1st column
 			sta	edCurX						; set it
 			lda	edCurY						; get current posY
 			cmpa	#7								; already at last one?
 			blo	IncRow						; no, skip adjustement
 			clra									; get first row					
 			sta	edCurY						; set it
 			bra	RghtExt						; apply change
IncRow	inc	edCurY						; point to next row
 			bra	RghtExt						; apply change
DoRght	inc	edCurX						; move to next column to the right
RghtExt 	lbsr	ShwPFnt						; show Position and Block number
 			lbra	WaitKey						; loop back to keyboard polling
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; OK - Moves to next char on the left
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
MovLeft	bsr	CondInv						; invert if still inverted
			lda	edCurX						; get current posX
 			bne	DoLeft						; not left end, do move left
 			lda	#31							; get last column
 			sta	edCurX						; set it
 			lda	edCurY						; get current posY
 			bne	DecRow						; not upper end, skip adjustement
 			lda	#7								; get last row					
 			sta	edCurY						; set it
 			bra	LeftExt						; apply change
DecRow	dec	edCurY						; point to previous row
			bra	LeftExt						; apply change
DoLeft	dec	edCurX						; move to next column to the left
LeftExt 	lbsr	ShwPFnt						; show Position and Block number
 			lbra	WaitKey						; loop back to keyboard polling
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; OK - moves to next char one row up
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
MovUp		bsr	CondInv						; invert if still inverted
			lda	edCurY						; get current posY
 			bne	DoUp							; not upper row, do move up
 			lda	#7								; get last possible row
 			sta	edCurY						; set it
 			bra	UpExt							; apply change
DoUp		dec	edCurY						; move to next row up
UpExt 	lbsr	ShwPFnt						; show Position and Block number
 			lbra	WaitKey						; loop back to keyboard polling
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; OK - moves to next char one row down
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
MovDown	bsr	CondInv						; invert if still inverted
 			lda	edCurY						; get current posY
 			cmpa	#7								; already at the last row?
 			blo	DoDown						; no, do move down
 			clra									; get 1st possible row
 			sta	edCurY						; set it
 			bra	UpExt							; apply change
DoDown	inc	edCurY						; move to next row down
DownExt 	lbsr	ShwPFnt						; show Position and Block number
 			lbra	WaitKey						; loop back to keyboard polling
; ----------------------------------------------------------------------------------------
; OK - inverts char if already inverted
; ----------------------------------------------------------------------------------------
CondInv	tst	invFlag						; is current block inverted?
			bne	Invert						; yes, revert to not inverted
 			rts									; else return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; OK - invert pointed char
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
Invert	ldx	#SCRNBEG						; point to screen beginning
			clra									; pepare 16 bits data
			ldb	edCurY						; get cursor Y position
			beq	CalDone						; if zero, no calculations needed
			tfr	d,y							; transfer value to regY
			ldd	#$0000						; reset accumulator 
CalPosY	addd	#384							; offset to next chrs row (12 rows x 32 chars)
			leay	-1,y							; decrement counter
			bne	CalPosY						; not done? loopback
CalDone	addb	edCurX						; add cursor X position
 			adca	#0								; propagate carry
 			leax	d,x							; point to 1st byte of that character
 			ldb	#12							; number of bytes to invert
1			com	,x								; invert one byte
 			leax	32,x							; point to byte below
 			decb									; decrement counter
 			bne	1b								; not yet done? loopback
 			com	invFlag						; invert flag	
 			ldx	#$0c00						; delay 
2			leax	-1,x							; decrement counter
 			bne	2b								; if not done, loopback
 			rts									; return
; ----------------------------------------------------------------------------------------
; OK - Draws a font to the screen.
; Params:	scrPtr	points to 1st screen byte to be used
;				regX		points to block data bytes to be used
; ----------------------------------------------------------------------------------------
DrawBloc	ldu	scrPtr						; point to 1st byte
			ldb	#12							; number of bytes to draw
NextByte	lda	,x+							; get a data byte
 			sta	,u								; put into screen
 			leau	32,u							; point to byte below
 			decb									; decrement counter
 			bne	NextByte						; not yet done?, loopback
 			ldu	scrPtr						; get used screen pointer
 			leau	1,u							; point to next char 1st byte
 			stu	scrPtr						; update pointer variable
 			rts									; return
; ------------------------------------------------------------------------------------------------------------------------------------------
; OK - Sets the graphic Mode and points to the graph page beginning
; ------------------------------------------------------------------------------------------------------------------------------------------
SetGMode	sta	$ffc0							; set SAM
			sta	$ffc3							;   for 
			sta	$ffc5							;   PMODE4
			lda	#$f8							; set PIA
			sta	SETPIA						;   to PM4 white palete
												; set page = $1e00/$200 = $f = %0001111
			sta	$ffd2							; bit6 = 0
			sta	$ffd0							; bit5 = 0
			sta	$ffce							; bit4 = 0
			sta	$ffcd							; bit3 = 1
			sta	$ffcb							; bit2 = 1
			sta	$ffc9							; bit1 = 1
			sta	$ffc7							; bit0 = 1
												; perform a PCLS
ClsScr	ldx	#SCRNBEG						; point to screen beginning
			ldd	#0								; get a zero word
NextWord	std	,x++							; clean two bytes
			cmpx	#SCREND						; already done?
			blo	NextWord						; no, loopback
			rts									; return
; ----------------------------------------------------------------------------------------
; OK - param regU points to char data to be printed
; 		  scrPtr points to the desired byte of screen
; ----------------------------------------------------------------------------------------
ShwText	lda	,u+							; get a char
 			beq	ShowTEx						; if end of text, exit
 			cmpa	#32							; is it space?
 			bne	1f								; no, skip adjustements
 			inc	scrPtr+1						; increment pointer low byte					
 			bne	ShwText						; if not zero, get next char
 			inc	scrPtr						; increment high byte
 			bra	ShwText						; skip it and get next one
1			ldx	#FONTAB						; point to Beginning of data block table
 			ldb	#12							; bytes per block
 			mul									; calculate offset
 			leax	d,x							; point to first byte of requested font
 			pshs	u								; save pointer
 			bsr	DrawBloc						; Draw that bloc
 			puls	u								; restore pointer
 			bra	ShwText						; go for next char
ShowTEx	rts									; return
; ----------------------------------------------------------------------------------------
; OK - Show ASCII char code
; ----------------------------------------------------------------------------------------
ShwPFnt	bsr	FillFnt						; fill numeric field for Fnt
 			ldx	#SCR23L						; point to L23
 			stx	scrPtr						; save to pointer
 			ldu	#txtFnt						; point to message
 			bsr	ShwText						; show string
 			rts									; return
; ----------------------------------------------------------------------------------------
; OK - fills ASCII code for selected font
; ----------------------------------------------------------------------------------------
FillFnt	lda	edCurY						; get cursor Y position (row)
 			ldb	#32							; get number of chars per column
 			mul									; calculate offset for this row
 			addb	edCurX						; add actual cursor X position (column)
			tfr	b,a							; use low byte
 			ldy	#tFntN						; point to hundreds in text string
 			bsr	Fill3Dig						; calculate digits
 			rts									; return
; ----------------------------------------------------------------------------------------
; OK - fills string with the number represented by regA into string pointed by regY
; here we need THREE digits for ascii codes 100-255
; ----------------------------------------------------------------------------------------
Fill3Dig ldb	#'0'							; put a zero
			stb	,y								; in hundreds
			stb	1,y							; and in tens
CalcHund	cmpa	#100							; is greater than 99?
			blo	PrepTens						; no, calculate tens
			suba	#100							; subtract one hundred
			inc	,y								; increment number of hundreds on screen
			bra	CalcHund						; back for more hundreds
PrepTens leay	1,y							; point to tens
CalcTens	cmpa	#10							; is greater than 9?
			blo	PutUnits						; no, so are the units
			suba	#10							; deduct 10 from parameter
			inc	,y								; increment number of tens on screen
			bra	CalcTens						; back for more tens
PutUnits	adda	#$30							; convert units to ASCII
			sta	1,y							; update units
			rts									; return
; ----------------------------------------------------------------------------------------
; VERIFY IT - Edit on a PMODE4 window - set Window according to the position of edited font
; ----------------------------------------------------------------------------------------
FntEdi4	pshs	d,x,y,u						; save workig registers
 			clra									; get a zero
 			sta	wInvFlag						; reset window flag
 			sta	wEdCurX						; X = 0
 			sta	wEdCurY						; Y = 0
 			ldx	#SCRNBEG						; point to screen beginning
 			leax	768,x							; skip 2 fonts rows (2x12=24rowsx32=768)
												; choose contrary side with respect to the char to edit
			lda	edCurX						; get cursor X position
			cmpa	#16							; is it at the right side of screen?
			bhi	UseLeft						; yes, skip next two
UseRight ldb	#20							; use the right part of screen
			bra	AppOrig						; skip next
UseLeft	clrb									; use the left part of screen
AppOrig 	abx									; use selected side
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 												; create a window with bg colour
DrawBg4	stx	wTopLeft						; save top left corner of edit window
 			lda	#120							; 120 rows (12+96+12)
1			ldb	#6								; 6 words for 12 columns (1+8+1+1+1) - borders plus big and small images
			ldu	#%0000000000000000		; set a word wth bg colour (black by default)
2 			stu	,x++							; paint 2 bytes
 			decb									; decrement column counter
 			bne	2b								; not done? loopback
 			leax	20,x							; point to row below (+12+20=32)
 			deca									; decrement row counter
 			bne	1b								; not done? loopback
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 												; create the real size image on the right
 												; and the extended block (edit area)
 			ldx	#SCRNBEG						; get screen beginning
			clra									; to create a 16 bits value
			ldb	edCurY						; get cusor Y position
			beq	CalDoneE						; if zero, no calculations needed
			tfr	d,y							; transfer value to regY
			ldd	#$0000						; reset accumulator
CalPosYE	addd	#384							; add offset to next chrs row (12 rows x 32 bytes)
			leay	-1,y							; decrement counter
			bne	CalPosYE						; not yet done, loopback
CalDoneE	addb	edCurX						; add cursor X position
 			adca	#0								; propagate carry
 			leax	d,x							; point to first byte of this char on screen
 			ldu	wTopLeft						; get edit window top-left corner			
 			leau	$0101,u						; beginning of extended font
 			ldy	wTopLeft						; get edit window top-left corner			
 			leay	$010a,y						; beginning of actual size font
 			ldb	#12							; number of bytes to copy (12 rows x 1 byte each)
3			lda	,x								; get a byte from screen
 			sta	,y								; put in window
 			lbsr	FillExt4						; fill extended area
 			leax	32,x							; point to byte below (source area)
 			leay	32,y							; point to byte below (edit area)
 			leau	384,u							; point to next char (row below)
 			decb									; decrement counter
 			bne	3b								; not done? loopback
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 												; wait for edit commands
WfKey4	lbsr	WinInv4						; invert pixel
 			jsr	[$A000]						; wait for commands (LRUD Break colour1-2-3-4 Space Apply1 ApplyAll) 
 			beq	WfKey4						; no key? lopback
 			cmpa	#9								; requested RIGHT?
 			beq	MovRght4						; process right
 			cmpa	#8								; requested LEFT?
 			lbeq	MovLeft4						; process left
 			cmpa	#94							; requested UP?	
 			lbeq	MovUp4						; process up
 			cmpa	#10							; requested DOWN?
 			lbeq	MovDown4						; process down
 			cmpa	#3								; was BREAK?
 			beq	BlkEdEx4						; exit EDITION mode
 			cmpa	#'A'							; was comand APPLY?
 			lbeq	AplyMod4						; yes, process it
 			cmpa	#'1'							; ASCII key lower than 1?
 			blo	WfKey4						; yes, discard it
 			cmpa	#'2'							; ASCII key greater than 2?
 			bhi	WfKey4						; yes, discard it
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
PaintP4	suba	#'1'							; convert ASCII to binary (0-1)
 			pshs	a								; save this key value
 			lbsr	CondWIn4						; invert pixel if required
 			lbsr	CalcPos4						; point to current pixel
 			ldu	#pixDat4						; point to table
 			lda	,s								; restore key value (0-1)
 			lda	a,u							; get value to put in extended image
 			ldb	#12							; 12 rows x pixel
1 			sta	,x								; set new colour
 			leax	32,x							; move pointer to row below
 			decb									; decrement counter
 			bne	1b								; not done? loopback
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 												; point to current row 1st byte in actual size font
 			ldx	wTopLeft						; point to top left byte of window
 			leax	$010a,x						; point to top byte in actual size font
 			lda	wEdCurY						; get Y coordinate (row num)
 			ldb	#32							; bytes per row
 			mul									; calculate offset
 			leax	d,x							; point to affected byte
 			lda	,x								; get current value for that byte
 			ldb	wEdCurX						; get pixel num (range between 0 and 7)
 			ldu	#maskW4						; point to masks
 			anda	b,u							; mask out pixel
			tst	,s								; was 0 (reset) pressed?
			beq	1f								; skip setting bit
			leau	8,u							; point to set value
			ora	b,u							; set 
1 			sta	,x								; update screen byte in actual size font
 			leas	1,s							; clean stack
 			bra	WfKey4						; loopback
BlkEdEx4	lbsr	ClsScr						; clean screen
 			puls	d,x,y,u,pc					; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
pixDat4	fcb	%00000000,%11111111		; byte values for colors
maskW4	fcb	$7f,$bf,$df,$ef
			fcb	$f7,$fb,$fd,$fe			; to mask out a pixel
setPix4	fcb	$80,$40,$20,$10
			fcb	$08,$04,$02,$01			; value to set pixel
; ----------------------------------------------------------------------------------------
MovRght4	bsr	CondWIn4						; invert pixel if required
 			lda	wEdCurX						; get edit cursor X
 			cmpa	#7								; is it at right end?
 			lbeq	WfKey4						; yes, reject command
 			inca									; else move cursor
 			bra	ApplyX4						; apply change
; ----------------------------------------------------------------------------------------
MovLeft4 bsr	CondWIn4						; invert pixel if required
 			lda	wEdCurX						; get edit cursor X
 			lbeq	WfKey4						; if at left end, reject command
 			deca									; else move cursor
ApplyX4	sta	wEdCurX						; apply change
 			lbra	WfKey4						; wait for another command
; ----------------------------------------------------------------------------------------
MovUp4	bsr	CondWIn4						; invert pixel if required
 			lda	wEdCurY						; get edit cursor Y
 			lbeq	WfKey4						; if at top end, reject command
 			deca									; else move cursor
 			bra	ApplyY4						; apply change
; ----------------------------------------------------------------------------------------
MovDown4	bsr	CondWIn4						; invert pixel if required
 			lda	wEdCurY						; get edit cursor Y
 			cmpa	#11							; is at bottom end? 
 			lbeq	WfKey4						; yes, reject command
 			inca									; else move cursor
ApplyY4	sta	wEdCurY						; apply change
 			lbra	WfKey4						; wait for another command
; ----------------------------------------------------------------------------------------
; Update loaded data from disk in RAM
; ----------------------------------------------------------------------------------------
AplyMod4	bsr	CondWIn4						; invert pixel if required
 												; point to beginning of edited block in table
 			ldx	#FONTAB						; point to font table beginning
 			ldb	edCurX						; add actual X
 			lda	#12							; bytes per font
 			mul									; so that regB contains offset on units
 			leax	d,x							; point to correct column
			clra									; to have a 16 bits value
			ldb	edCurY						; get cursor Y position
			beq	CalDoneA						; if zero no calculations needed
			tfr	d,y							; transfer to regY
			ldd	#$0000						; reset accumulator
CalPosYA	addd	#384							; add offset to next char (12 x 32)
			leay	-1,y							; decrement counter
			bne	CalPosYA						; not yet done? loopback
CalDoneA	leax	d,x							; move pointer to row beginning
 												; point to top byte of actual size block in edit window
 			ldu	wTopLeft						; get edit window top-left corner			
 			leau	$010a,u						; beginning of actual size block
 												; copy a whole char
 			ldb	#12							; number of bytes to copy
1			lda	,u								; get a byte from edited area
 			sta	,x+							; update block table
 			leau	32,u							; move pointer to byte below
 			decb									; decrement counter
 			bne	1b								; not done? loopback
 			lbra	BlkEdEx4						; back to choose another font
; ----------------------------------------------------------------------------------------
; OK - Inverts char if not alreday inverted
; ----------------------------------------------------------------------------------------
CondWIn4	tst	wInvFlag						; is current pixel inverted?
 			bne	WinInv4						; yes, revert to not inverted
 			rts									; else return
WinInv4	bsr	CalcPos4						; calculate top byte for current pixel
 			ldb	#12							; rows to invert
1 			com	,x								; invert one row
 			leax	32,x							; pointto row below
 			decb									; decrement counter
 			bne	1b								; not done? loopback
 			com	wInvFlag						; invert flag
 			ldx	#$0c00						; delay
1			leax	-1,x							; decrement counter
 			bne	1b								; not done, loopback
 			rts									; return
; ----------------------------------------------------------------------------------------
CalcPos4	ldx	wTopLeft						; point to top byte
 			leax	$0101,x						; skip 1 row and 1 column
			clra									; to have a 16 bits value
			ldb	wEdCurY						; get cursor Y position
			beq	CalDoneP						; if zero no calculations needed
			tfr	d,y							; transfer to regY
			ldd	#$0000						; reset accumulator
CalPosYP	addd	#384							; add offset to next char (12 x 32)
			leay	-1,y							; decrement counter
			bne	CalPosYP						; not yet done? loopback
CalDoneP	addb	wEdCurX						; get 1st column
 			adca	#0								; propagate carry
 			leax	d,x							; move pointer to row beginning
 			rts									; return
; ----------------------------------------------------------------------------------------
; VERIFY IT - Fills one byte in 12 rows
; params: regU points to top byte to be drawn
;			 regA	contains data byte to show
; ----------------------------------------------------------------------------------------
FillExt4	pshs	a,b,x,y,u					; save registers
 			bsr	F4Pix7B4						; put pixel 1 (left) in 12 rows
 			lda	,s								; get received value
 			bsr	F4Pix6B4						; put pixel 2 in 12 rows
 			lda	,s								; get received value
 			bsr	F4Pix5B4						; put pixel 3 in 12 rows
 			lda	,s								; get received value
 			bsr	F4Pix4B4						; put pixel 4 (right) in 12 rows
 			lda	,s								; get received value
 			bsr	F4Pix3B4						; put pixel 5 (left) in 12 rows
 			lda	,s								; get received value
 			bsr	F4Pix2B4						; put pixel 6 in 12 rows
 			lda	,s								; get received value
 			bsr	F4Pix1B4						; put pixel 7 in 12 rows
 			lda	,s								; get received value
 			bsr	F4Pix0B4						; put pixel 8 (right) in 12 rows
 			puls	a,b,x,y,u,pc				; restore registers and return
; ----------------------------------------------------------------------------------------
F4Pix7B4	lsra									; move bit7 7 times to the right
F4Pix6B4	lsra									; move bit7-6 6 times to the right
F4Pix5B4	lsra									; move bit7-6-5 5 times to the right
F4Pix4B4	lsra									; move bit7-6-5-4 4 times to the right
F4Pix3B4	lsra									; move bit7-6-5-4-3 3 times to the right
F4Pix2B4	lsra									; move bit7-6-5-4-3-2 2 times to the right
F4Pix1B4	lsra									; move bit7-6-5-4-3-2-1  times to the right
F4Pix0B4	anda	#$01							; always use bit 0
 			ldx	#pixDat4						; point to colour values
 			lda	a,x							; get coresponding value
			ldb	#12							; rows to draw
			pshs	u								; save screen pointer
1 			sta	,u								; put in top row
 			leau	32,u							; point to next row
 			decb									; decrement counter
 			bne	1b								; not done, loopback
 			puls	u								; restore pointer
 			leau	1,u							; point to next byte (pixel)
 			rts									; return
; --------------------------------------------------------------------------------------------------------------------------------------------------------------------
edCurX	fcb	$00							; cursor for edit mode
edCurY	fcb	$00							; cursor for edit mode
invFlag	fcb	$00							; inverted cursor flag 
scrPtr	fdb	$0000							; 1st byte to be drawn
txtFnt	fcc	/ FNT /						; text block
tFntN		fcn	/000/							; block number ... 3 chars
wTopLeft	fdb	$0000							; edit window top left corner
wEdCurX	fcb	$00							; cursor for edit mode
wEdCurY	fcb	$00							; cursor for edit mode
wInvFlag	fcb	$00							; inverted cursor flag 
; ----------------------------------------------------------------------------------------
EndPgm	equ	*
