; ---------------------------------------------------------------------------------------------------------------
; TOOLS59 - 2019-11-18 - Pere Serrat 	-	A set of tools to deal with CoCo-VGA
; ---------------------------------------------------------------------------------------------------------------
			ORG	$4A00
			PUT	$4A00
; ---------------------------------------------------------------------------------------------------------------
; structure for VGA config area and structure for AMC2 boards. MUST begin in a 512 bytes boundary address
; ---------------------------------------------------------------------------------------------------------------
vgaRegs	equ	$1000							; $88 bytes - area to be used to configure VGA. Mandatory for D32
vgaEnd	equ	$1200							; 1st byte out of that 'page' area
SLOTRST	equ	$00							; offset for reset
SLOTEDT	equ	$01							; offset for edit mask
SLOTFNT	equ	$03							; offset for font
SLOTART	equ	$04							; offset for artifact
SLOTEXT	equ	$05							; offset for extras
SLOTENH	equ	$08							; offset for enhanced modes
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SLOTBLK	equ	$20							; offset substitute for color BLACK
SLOTGRN	equ	$22							; offset substitute for color GREEN
SLOTYLW	equ	$24							; offset substitute for color YELLOW
SLOTBLU	equ	$26							; offset substitute for color BLUE
SLOTRED	equ	$28							; offset substitute for color RED
SLOTBUF	equ	$2a							; offset substitute for color BUFF
SLOTCYA	equ	$2c							; offset substitute for color CYAN
SLOTMAG	equ	$2e							; offset substitute for color MAGENTA
SLOTORG	equ	$30							; offset substitute for color ORANGE
SLOTDGR	equ	$32							; offset for color dark Green
SLOTDOG	equ	$34							; offset for color dark Orange
SLOTFGT	equ	$80							; offset for text Foreground 
SLOTBGT	equ	$82							; offset for text Background 
SLOTBOR	equ	$84							; offset for Border 
SLOTSCL	equ	$86							; offset for Scanlines
VGALEN	equ	$88							; area length
; ---------------------------------------------------------------------------------------------------------------
; structure for palete data. The palete file *MUST* be loaded here from disk
; ---------------------------------------------------------------------------------------------------------------
wrkPal	equ	$1300							; 30 bytes reserved to read-write palete file
wrkNam	equ	$131E							; game name (8 chars)
wrkDesc	equ	$1326							; game description (20 bytes)
palEnd	equ	$1339							; end of area
palLen	equ	palEnd-wrkPal				; value for clearing area (58 bytes)
; - - - - - - - -	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PALBLK	equ	00								; 2 bytes for color BLACK
PALGRN	equ	02								; 2 bytes for color GREEN
PALYLW	equ	04								; 2 bytes for color YELLOW
PALBLU	equ	06								; 2 bytes for color BLUE
PALRED	equ	08								; 2 bytes for color RED
PALBUF	equ	10								; 2 bytes for color BUFF
PALCYA	equ	12								; 2 bytes for color CYAN
PALMAG	equ	14								; 2 bytes for color MAGENTA
PALORG	equ	16								; 2 bytes for color ORANGE
PALBOR	equ	18								; 2 bytes for color BORDER
PALDGR	equ	20								; 2 bytes for color dark Green
PALDOG	equ	22								; 2 bytes for color dark Orange
PALFGT	equ	24								; 2 bytes for text Foreground 
PALBGT	equ	26								; 2 bytes for text Background 
PALSCL	equ	28								; 2 bytes for Scanline
PALDUM	equ	30								; 2 unused bytes to round up count
													; for a total length of 32 bytes
; ---------------------------------------------------------------------------------------------------------------
INTDIS	equ	$50							; value to disable interrupts
INTENA	equ	$af							; value to enable interrupts
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
SCRBEG	equ	$0400							; text screen beginning
SCREND	equ	$0600							; text screen end
PIA0BCTL	equ	$ff03							; PIA0 side B control register
PIA1BDAT	equ	$ff22							; PIA1 sideB data register
SAMADDRS	equ	$ffc6							; 1st address for SAM to define working page in RAM
; ===============================================================================================================

; ===============================================================================================================
												; Jump table to access the functions
L4A00		jmp	RstPal						; restores CoCo-VGA to default values
L4A03		jmp	ChgPal						; copies palette loaded dat to VGA area and send it to CoCo-VGA
L4A06		jmp	ChgChr						; send to CoCo-VGA the loaded charset
L4A09		jmp	ShwDsc						; shows game descriptions on screen
L4A0C		jmp	InvOpt						; invert selected option
L4A0F		jmp	ChgCol						; change color for SG blocks
L4A12		jmp	ErrTrap						; trap errors
L4A15		fcb	$00							; parameter for Printing descriptions
CURRPAGE	equ	L4A15							; current page being shown, filled by caller
CURROPT	equ	L4A15							; user game choice
; ===============================================================================================================

; ===============================================================================================================
; This part resets the module to the default values
; ===============================================================================================================
												; Step 1 - fill area to configure CoCoVGA
RstPal	lbsr	ClearVga						; copy palette data to VGA area
			lda	#$ff							; set all features to be set to default values
			sta	vgaRegs						; put in VGA area
			bra	ApplyMod
; ===============================================================================================================

; ===============================================================================================================
; This part changes the palette colors sending the loaded data to the module 
; ===============================================================================================================
ChgPal	lbsr	CleanVga						; clean VGA area
			lbsr	RstModul						; reset module
			lbsr	CopyPal						; copy LOADED palette data to VGA area
			bsr	ApplyMod						; apply loaded palette
			rts									; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 2 - Wait for VSYNC, which signals the start of the vertical blanking region.
ApplyMod	pshs	cc     						; save CC
			orcc	#INTDIS  					; mask interrupts
			ldu	#PIA0BCTL					; pointer for PIA0 B-side
			lda	,u	 							; $ff03 - get PIA config
			pshs	a      						; save PIA configuration 
			ora	#$04   						; ensure PIA 0B is NOT setting direction
			sta	,u								; $ff03 - update value
			anda	#$fd   						; vsync flag - trigger on falling edge
			sta	,u								; $ff03 - update value
			ora	#$01   						; enable vsync IRQ (although interrupt itself will be ignored via mask)
			sta	,u								; $ff03 - update value
			lda	-1,u  						; $ff02 - clear any pending VSYNC
L01		lda	,u		  						; $ff03 - wait for flag to indicate...
			bpl	L01     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 3 - During VSYNC, point SAM to 512 byte page set up in step 1 (via SAM page select registers $FFC6-$FFD3)
 			ldd	#vgaRegs						; point to VGA data area beginning
 			bsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 4 - Also during VSYNC, write 6847 VDG combination lock via PIA1B ($FF22) bits 7:3.
Step4		ldy	#PIA1BDAT					; pointer for PIA1 B-side
			lda	,y  							; $ff22 - get current PIA value
			anda	#%00000111					; mask off bits to change
			ora	#%10010000					; $90 - set combo lock 1
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%01001000					; $48 - set combo lock 2
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%10100000					; $a0 - set combo lock 3
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%11111000 					; $f8 - set combo lock 4
			sta	,y	  							; $ff22 - write to PIA
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 5 - Still during VSYNC, configure VDG and SAM back to mode 0 
												; In this case, the desired CoCoVGA register page to program is 0
			lda	,y	  							; $ff22 - get current PIA value
			anda	#%00000111					; mask off bits to change
			ora	#%00000000					; select CoCoVGA register page 0
			sta	,y	  							; $ff22 - write to PIA
			sta	-6,x  						; $ffc0 - clear SAM_V0	0
			sta	-4,x  						; $ffc2 - clear SAM_V1	0
			sta	-2,x  						; $ffc4 - clear SAM_V2	0	-> 000
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 6 -  Wait for next VSYNC while SAM and VDG stream in the entire page of register values 
 												; to CoCoVGA and CoCoVGA displays the previous frame of video
L02		lda	,u								; $ff03 - wait for flag to indicate...
			bpl	L02     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
			puls	a      						; from stack... 
			sta	-1,u	 						; $ff02 - ...restore original PIA configuration
			puls	cc     						; restore ability to see interrupts
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 7 - Point SAM page select to video page you want to display
 			ldd	#SCRBEG						; point to text screen beginning
 			bsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 8 - Program SAM and VDG to the appropriate video mode. 
 												; For 32x16 text and graphics, GM is 0 and A/G is 0.
			lda	,y		  						; $ff22 - get current PIA value
			anda	#$0f  						; mask off bits to change
UseCss0	anda	#%11110111					; reset bit3 to ensure color palete 0
PutCss	sta	,y		  						; $ff22 - write to PIA
			sta	-6,x  						; $ffc0 - clear SAM_V0
			sta	-4,x  						; $ffc2 - clear SAM_V1
			sta	-2,x  						; $ffc4 - clear SAM_V2	-> 000
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; set SAM memory address properly. regD must contain ADDRESS
; ---------------------------------------------------------------------------------------------------------------
SetSAM	ldx	#SAMADDRS					; $ffc6 - pointer for SAM Display Offset
			asra									; use high byte divided by two = regD/512
SamF0		bita	#%00000001					; is bit 0 set?
			bne	SetF0							; yes, setF0
			stb	,x		  						; $c6 - clear 	SAM_F0
			bra	SamF1							; skip next
SetF0		stb	1,x	  						; $c7 - set 	SAM_F0
SamF1		bita	#%00000010					; is bit 1 set?
			bne	SetF1							; yes, setF1
			stb	2,x	  						; $c8 - clear	SAM_F1
			bra	SamF2							; skip next
SetF1		stb	3,x	  						; $c9 - set 	SAM_F1
SamF2		bita	#%00000100					; is bit 2 set?
			bne	SetF2							; yes, setF2
			stb	4,x	  						; $ca - clear 	SAM_F2
			bra	SamF3							; skip next
SetF2		stb	5,x	  						; $cb - set 	SAM_F2
SamF3		bita	#%00001000					; is bit 3 set?
			bne	SetF3							; yes, setF3
			stb	6,x	  						; $cc - clear 	SAM_F3
			bra	SamF4							; skip next
SetF3		stb	7,x	  						; $cd - set 	SAM_F3
SamF4		bita	#%00010000					; is bit 4 set?
			bne	SetF4							; yes, setF4
			stb	8,x	  						; $ce - clear 	SAM_F4
			bra	SamF5							; skip next
SetF4		stb	9,x	  						; $cf - set 	SAM_F4
SamF5		bita	#%00100000					; is bit 5 set?
			bne	SetF5							; yes, setF5
			stb	10,x  						; $d0 - clear 	SAM_F5
			bra	SamF6							; skip next
SetF5		stb	11,x  						; $d1 - set 	SAM_F5
SamF6		bita	#%01000000					; is bit 6 set?
			bne	SetF6							; yes, setF6
			stb	12,x  						; $d2 - clear 	SAM_F6
			rts									; return
SetF6		stb	13,x  						; $d3 - set 	SAM_F6
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; Copy data from Palette to VGA and set correct params
; ---------------------------------------------------------------------------------------------------------------
CopyPal	bsr	Pal2Vga						; copy data to VGA area
			ldx	#vgaRegs						; point to VGA area
			lda	#$2c							; set bits 5,3,2 
			ora	#%10000001					; set bits 7,0 
			sta	SLOTEDT,x					; in param

			lda	#%00000100					; set bit 2
			sta	SLOTFNT,x					; update font param
			lda	#%00000100					; set bit 2
			sta	SLOTENH,x					; update enhanced modes param

			lda	SLOTEXT,x					; get 'extras' param

			tst	SLOTBOR,x					; is border enabled?
			bpl	NoBorder						; no, skip next one
			ora	#$03							; enable border color			
NoBorder	ora	#$08							; enable quiet border status
			tst	SLOTSCL,x					; are scanlines enabled?
			bpl	NoScanL						; no, skip next two
			ora	#$04							; enable scanlines
NoScanL	tst	SLOTFGT,x					; is foreground text enabled?
			bne	EnaTxtC						; yes, enable it
			tst	SLOTBGT,x					; is background text enabled?
			beq	LfEna							; no, skip next
EnaTxtC	ora	#$80							; enable text colors
LfEna		sta	SLOTEXT,x					; update param
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; send to CoCo-VGA a page with just the reset byte to $ff
; ---------------------------------------------------------------------------------------------------------------
RstModul	lda	#$ff							; set all features to be set to default values
			sta	vgaRegs						; put in VGA area
			lbsr	ApplyMod						; send page to module
			bsr	CleanVga						; reset VGA area all bytes
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; clear (reset) all bytes in VGA area
; ---------------------------------------------------------------------------------------------------------------
CleanVga	bsr	ClearVga						; clean the whole page
			ldx	#vgaRegs						; point to VGA area
			clr	,x								; clear reset byte
			clr	SLOTEDT,x					; clear reset byte
			ldd	#$0D48						; regA: 00001110 -> artifacts
													; regB: 01001000 -> text palette,quiet border,border color mode
			std	SLOTART,x					; put in VGA area
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
ClearVga	ldx	#vgaRegs						; point to VGA data area
			clra									; set a zero
			clrb									; 16 bits long
ClVga01	std	,x++							; reset a word
			cmpx	#vgaEnd						; got to the end?
			blo	ClVga01						; no, loopback
			rts									; return
; ---------------------------------------------------------------------------------------------------------------
; copy palette area to VGA area
; ---------------------------------------------------------------------------------------------------------------
Pal2Vga	ldu	#wrkPal						; point to loaded palette area
			ldy	#vgaOff						; point to VGA offset table
			clr	,-s							; create and reset a counter in stack
Pal2V01	ldb	,s								; get counter from stack
			cmpb	#15							; got to the end?
			bhs	Pal2VEx						; yes, exit
			ldb	b,y							; get VGA offset for this item
			clra									; make 16 bits offset
			ldx	#vgaRegs						; point to VGA beginning area
			leax	d,x							; point to destination item data (unsigned addition)
			ldd	,u++							; get 2 bytes from palette and move pointer 
			std	,x								; copy to VGA area
			inc	,s								; increment counter
			bra	Pal2V01						; loopback
Pal2VEx	puls	b,pc							; clean stack and return
; ---------------------------------------------------------------------------------------------------------------
; offsets for individual slots in VGA area - indexed by item on main menu screen
; ---------------------------------------------------------------------------------------------------------------
vgaOff	fcb	32								; VGA area offset for Black
			fcb	34,36,38,40					; VGA area offset for Green,Yellow,Blue,Red
			fcb	42,44,46,48					; VGA area offset for Buff,Cyan,Magenta,Orange
			fcb	132							; VGA area offset for Border
			fcb	50,52							; VGA area offset for darkGreen,darkOrange
			fcb	128,130,134					; VGA area offset for textForeground,background,scanline
; ===============================================================================================================

; ===============================================================================================================
; This part changes the character set sending the loaded one to the module
; ===============================================================================================================
ChgChr	pshs	cc     						; save CC
			orcc	#INTDIS  					; mask interrupts
			ldu	#PIA0BCTL					; pointer for PIA0 B-side

			lda	-1,u							; $ff02 - clear any pending vsync flag
L03		lda	,u								; $ff03 - wait for flag to indicate...
			bpl	L03     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
												; set page for new charset
chrPag	lda	#$0e							; charset is LOADED at $e00, so $e00/512=7 (doubled = 14d=$0e)
 			lbsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
												; program the cocovga combo lock
			ldy	#PIA1BDAT					; pointer for PIA1 B-side
			lda	,y  							; $ff22 - get current PIA value
			pshs	a								; save onto stack
			anda	#%00000111					; mask off bits to change
			ora	#%10010000					; $90 - set combo lock 1
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%01001000					; $48 - set combo lock 2
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%10100000					; $a0 - set combo lock 3
			sta	,y  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%11111000 					; $f8 - set combo lock 4
			sta	,y	  							; $ff22 - write to PIA
			anda	#%00000111					; mask off bits to change
			ora	#%11000000 					; $c0 - SET REGISTER BANK $18 FOR COCOVGA CHAR SET UPLOAD (CG3) - NEW
			sta	,y	  							; $ff22 - write to PIA
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
												; program SAM to also work in cg3 - new
			sta	-6,x  						; $ffc0 - clear SAM_V0	0
			sta	-4,x  						; $ffc2 - clear SAM_V1	0
			sta	-1,x  						; $ffc5 - set   SAM_V2	0	-> 100
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
												; wait for next vsync so cocovga can process data from the
												; current video page
L04		lda	,u								; $ff03 - wait for flag to indicate...
			bpl	L04     						; ...falling edge of FS (Field Sync)
			lda	-1,u  						; $ff02 - clear VSYNC interrupt flag
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			puls	a								; get value saved from stack
			sta	,y								; restore pia state and return to text mode
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 												; Step 7 - Point SAM page select to video page you want to display
 			ldd	#SCRBEG						; point to text screen beginning
 			lbsr	SetSAM						; set SAM to access this area
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
												; program sam to also work in alphanum/sg4 - new
			sta	-6,x  						; $ffc0 - clear SAM_V0
			sta	-4,x  						; $ffc2 - clear SAM_V1
			sta	-2,x  						; $ffc4 - clear SAM_V2	-> 000

			puls	cc     						; restore ability to see interrupts
			rts									; return
; ===============================================================================================================

; ===============================================================================================================
; This part prints on the screen 12 game descriptions. Needs the page number loaded into variable: currPage
; ===============================================================================================================
; structure for the Packed file with multiple palettes
; ---------------------------------------------------------------------------------------------------------------
MULTIPAL	equ	$6000							; whole packed file with 'n' palettes
PALLEN	equ	$3A							; length of an entry (one palette) - 58 bytes							
; ---------------------------------------------------------------------------------------------------------------
FIRSTROW	equ	$400+32*2+1					; second line, second column
NUMROWS	equ	12								; max number of games on screen
NUMCHARS	equ	20								; description length
DATANAME	equ	38								; data and name length
; ---------------------------------------------------------------------------------------------------------------
ShwDsc	lda	CURRPAGE						; get received page number
			beq	ShwExit						; if zero exit
			cmpa	#10							; greater than 9?
			bhs	ShwExit						; yes,reject it and exit
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
Process	deca									; offset zero (page 1 = offset 0) number will be between 0-8
			ldb	#NUMROWS						; number of games per page = 12
			mul									; number of registers to skip - result 0 to 96 in regB
												; now convert to bytes (58 bytes per palette)
			lda	#PALLEN						; get length of a palette
			mul									; multiply - result in regD
			ldx	#MULTIPAL					; point to multipalette beginning
			leax	d,x							; point to desired 1st palette beginning
			leax	DATANAME,x					; skip data and name
			ldu	#FIRSTROW					; point to 1st char on screen
			clrb									; previous char for first option in lowercase
			stb	charOpt						; save to variable
			lda	#NUMROWS						; max number of rows to fill
			pshs	a								; save on stack
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
AddOpt	pshs	x,u							; save pointers
			inc	charOpt						; increment letter to be shown
			lda	charOpt						; get it
			ldb	#122							; ':'
			std	,u++							; set choice
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
			ldb	#NUMCHARS					; number of chars to be poked
ShowLine	lda	,x+							; get one char from packed file
			bne	UseIt							; if zero, go for next description
			lda	#' '							; get a space if it was zero
UseIt		ora	#64							; ensure green space and uppercase
			sta	,u+							; put on screen
			decb									; decrement counter
			bne	ShowLine						; not yet done? loopback
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
NextLine	puls	x,u							; get pointers
			leau	32,u							; point to next row
			leax	PALLEN,x						; point to next description
			dec	,s								; decrement row counter
			ldd	-DATANAME,x					; get 2 bytes from palette beginning
			cmpd	#$aabb						; is end of palette?
			beq	ClUnused						; yes, go clean unused lines
			tst	,s								; is counter equal zero?
			bne	AddOpt						; no, loopback
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
ClUnused	lda	,s								; get counter from stack
			beq	CleanStk						; no more lines to be filled
			ldd	#$6016						; get a space, 22 times (option,':' and 20 chars)
			pshs	u								; save pointer to screen
CleanLin	sta	,u+							; clean one char on screen
			decb									; decrement counter
			bne	CleanLin						; not yet done? loop
			puls	u								; get pointer
			leau	32,u							; point to next row
			dec	,s								; decrement counter in stack
			bra	ClUnused						; go for next row
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
CleanStk	leas	1,s							; get rid of counter in stack
ShwExit	rts									; return
; ---------------------------------------------------------------------------------------------------------------
charOpt	fcb	$00							; letter that has to be pressed to choose that game
; ===============================================================================================================

; ===============================================================================================================
; invert the user selected game description
; ===============================================================================================================
InvOpt	ldx	#$0443						; point to first description
			ldb	CURROPT						; get param (line)
			decb									; 1 = first row (no offset)
			lda	#32							; chars per line
			mul									; calculate offset
			leax	d,x							; point to text beginning
			ldb	#20							; number of chars to be inverted
Inv01		lda	,x								; get one char
			anda	#%10111111					; invert it resetting bit 6
			sta	,x+							; update on screen
			decb									; decrement counter
			bne	Inv01							; not yet done? loopback
			rts									; return
; ===============================================================================================================

; ===============================================================================================================
; this will change the colors of SG blocks. value base = 143
; big widget is BUFF, then the other four vertically are: CYAN, YELLOW, BLUE and RED
; offsets are:  +64         										 +80   +16     +32      +48
; ===============================================================================================================
ChgCol	ldx	#SCRBEG						; point to screen beginning
ChgC01	lda	,x+							; get a char
			bpl	NoSgVal						; if not semigraphic, test end 
			cmpa	#175							; lower than full blue? so it is yellow
			blo	ReWind						; yes, roll to cyan
			suba	#16							; step to previous color
			bra	ChgC02						; skip next
ReWind	adda	#64							; convert to cyan
ChgC02	sta	-1,x							; update screen
NoSgVal	cmpx	#SCREND						; got to the end?
			blo	ChgC01						; no, loop
			rts									; return
; ===============================================================================================================

; ===============================================================================================================
; sets the code needed to trap basic errors within the basic and when starting a game
; ===============================================================================================================
ErrTrap	leax   StdTrap,PCR					; get new Error routine address
			cmpx   $018f						; is already in ErrorMessage hook?	
			beq    ErrEx						; yes, exit
			ldd    $018f						; get current hook address
			stx    $018f						; put new address
			std    NewAdd,PCR					; save current value to RAM 
			lda    $018e						; get 1st byte from that hook
			sta    NewJmp,PCR					; save to RAM too
			lda    #$7e							; get JMP opcode
			sta    $018e						; put in front of new routine address
ErrEx		rts									; return
												; new Error routine
StdTrap	ldx    <$68							; get current Line number
			cmpx   #$ffff						; is it manual input?
			beq    NewJmp						; yes, exit
ErrT01	pshs   b								; save error code
			ldx    #$f9ff						; get value 63999 (line number to be called)
			stx    <$2b							; put in variable for GOTO command
	IF COCO
			jsr    $ad01						; search for this line number 
	ELSE
			jsr    $83ff						; search for this line number 
	ENDIF
			puls   b								; restore error code
			bcs    NewJmp						; if *not* found exit
			lds    <$21							; get stack address
	IF COCO
			jsr    $aea9						; GOTO that special line
			jmp    $ad9e						; enter the Basic Interpreter
	ELSE
			jsr    $85dc						; GOTO that special line
			jmp    $849f						; enter the Basic Interpreter
	ENDIF
NewJmp	rts									; self-modified to have a jmp opcode
NewAdd	rts									; selfmodified to have
			rts									; the system Error hook routine
			ldd    ,s							; get stacked address
			addd   #$0004						; add 4
			std    ,s							; update in stack
			rts									; return
												; special entry point for ml error
SpeTrap	ldx	 <$76							; 9E 76	; get saved by Basic $a6 pointer
			stx	 <$a6							; 9F A6	; restore it
			bra	 ErrT01						; 20 D8	; enter routine skipping mode test
; ===============================================================================================================
												; this part ensures Drivewire compatibility
	IF COCO
		IF ((*+10)%256 > 0)					; if not multiple of 256
				rzb	 256-((*+10)%256)		; fill to make it DW4 compatible
		ENDIF										; end
	ELSE
		IF ((*+9)%256 > 0)					; if not multiple of 256
				rzb	 256-((*+9)%256)		; fill to make it DW4 compatible
		ENDIF										; end
	ENDIF
; ------------------------------------------------------------------------------------------
PgmEnd	equ	*
